from json.decoder import JSONDecodeError
from json import loads as json_loads
from json import dumps as json_dumps

from channels.generic.websocket import AsyncWebsocketConsumer


class NotifyConsumer(AsyncWebsocketConsumer):
    def __init__(self, *args, **kwargs):
        super().__init__(args, kwargs)
        self.room_group_name = "general"
        # self.event_slug = None

    async def connect(self):
        # self.event_slug = self.scope["url_route"]["kwargs"]["event_slug"]
        # self.room_group_name = f"chat_{self.event_slug}"

        # Join room group
        await self.channel_layer.group_add(self.room_group_name, self.channel_name)

        await self.accept()

    async def disconnect(self, close_code):
        # Leave room group
        await self.channel_layer.group_discard(self.room_group_name, self.channel_name)

    # Receive message from WebSocket
    async def receive(self, text_data):
        try:
            text_data_json = json_loads(text_data)
            message = text_data_json["message"]

            # Send message to room group
            await self.channel_layer.group_send(
                self.room_group_name, {"type": "chat.message", "message": message}
            )
        except JSONDecodeError:
            await self.send(text_data=json_dumps({"message": "error", "error": "malformed json"}))
        except KeyError as e:
            await self.send(text_data=json_dumps({"message": "error", "error": f"missing key: {str(e)}"}))
        except Exception as e:
            await self.send(text_data=json_dumps({"message": "error", "error": "unknown error"}))
            raise e

    # Receive message from room group
    async def chat_message(self, event):
        message = event["message"]

        # Send message to WebSocket
        await self.send(text_data=json_dumps({"message": message}))
